/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "config_data.h"

/******************************************************************************/

// This structure keeps an entry of the config file. Only use the kvp_xxx
// methods to access the structure as they perform memory management as well.
//
struct kvp
{
	char	*key;
	char	*value;
};


#define KVP_MAX_LEN	256

/******************************************************************************/

static list_t *kvp_list = 0;

/******************************************************************************/

#define kvp_get_key(_p)			(_p)->key
#define kvp_get_value(_p)		(_p)->value

/******************************************************************************/

static void kvp_change_value(struct kvp *pair, const char *value)
{
	assert(pair && value);

	if (pair->value) free(pair->value);
	pair->value = strdup(value);
}


/******************************************************************************/

static struct kvp *kvp_new(const char *key, const char *value)
{
	struct kvp	*pair;

	pair = (struct kvp *) malloc(sizeof(struct kvp));
	assert(pair);

	pair->key	= strdup(key);
	pair->value	= strdup(value);

	return pair;
}


/******************************************************************************/

static void kvp_delete(struct kvp *pair)
{
	// Ignore NULL calls
	if (!pair)	return;

	if (pair->key)		free(pair->key);
	if (pair->value)	free(pair->value);
	free(pair);
}


/******************************************************************************/

static struct kvp *kvp_find(const char *key)
{
	list_t	*iter;

	for (iter = list_first(kvp_list); iter; iter = list_next(iter))
	{
		struct kvp *pair;

		pair = list_data(iter);
		if (0 == strcmp(kvp_get_key(pair), key))
		{
			return pair;
		}
	}
	return 0;
}


/******************************************************************************
 *
 * config_data interface functions
 *
 ******************************************************************************/

int config_data_read(const char *filename)
{
	FILE *fp;
	char line[KVP_MAX_LEN];
	char *p;

	// If we already have a list, discard it...
	//
	if (kvp_list)
	{
		config_data_clear();
	}

	assert(filename);

	fp = fopen(filename, "r");
	if (!fp)
	{
		return -1;
	}

	do
	{
		p = fgets(line, sizeof(line), fp);

		if (p)
		{
			char *eq;

			// Truncate the line, just in case it was longer than
			// sizeof(line)
			//
			line[sizeof(line) -1] = '\0';


			// If the line does not contain a '=' or it begins with
			// a '#', we take the line as it is and do not treat it
			// as an entry.
			//
			if (('#' == line[0]) || (NULL == (eq = strchr(p, '='))))
			{
				// Sey the key to '#' so we can identify the
				// entry later when we write it to disk.
				//
				kvp_list = list_add(kvp_list, kvp_new("#", line));
				continue;
			}


			// There was a '=', let's create an entry...
			//
			// remove the \n at the end, if it exists
			//
			if ('\n' == line[strlen(line) - 1])
			{
				line[strlen(line) - 1] = '\0';
			}


			// Set the = to a \0, this will be our key
			//
			*eq = '\0';

			// Everything after that will be our value
			//
			eq++;

			// Add the key value pair to the list
			//
			kvp_list = list_add(kvp_list, kvp_new(p, eq));
		}
	}
	while (p);

	fclose(fp);
	return 0;
}


/******************************************************************************/

int config_data_write(const char *filename)
{
	FILE *fp;
	char line[KVP_MAX_LEN];

	list_t	*iter;

	// Do nothing, if we don't have a kvp_list
	//
	if (!kvp_list)
	{
		return 0;
	}

	assert(filename);

	fp = fopen(filename, "w");
	if (!fp)
	{
		return -1;
	}

	for (iter = list_first(kvp_list); iter; iter = list_next(iter))
	{
		int	rc;

		struct kvp	*pair = (struct kvp *) list_data(iter);

		// Check for non-entry lines. Non-entry lines have a '#' as the
		// key and the line as the value.
		//
		if (0 == strcmp(kvp_get_key(pair), "#"))
		{
			fputs(kvp_get_value(pair), fp);
			continue;
		}


		// brute force check, we must not have any key value pairs
		// longer than our buffer.
		//
		assert(sizeof(line) > 
			strlen(kvp_get_key(pair)) + strlen(kvp_get_value(pair)) + 4);

		sprintf(line, "%s=%s\n", kvp_get_key(pair), kvp_get_value(pair));

		rc = fputs(line, fp);
		if (EOF == rc) break;
	}

	fclose(fp);
	return 0;
}


/******************************************************************************/

void config_data_clear(void)
{
	list_t	*iter;
	
	// remove all the elements from the list and free the list.
	//
	for (iter = list_first(kvp_list); iter; iter = list_next(iter))
	{
		struct kvp *pair = list_data(iter);

		kvp_delete(pair);
	}
	list_free(kvp_list);
	kvp_list = 0;
}


/******************************************************************************/

const char *config_data_get(const char *key)
{
	struct kvp *pair;

	pair = kvp_find(key);

	return pair ? kvp_get_value(pair) : 0;
}


/******************************************************************************/

void config_data_set(const char *key, const char *value)
{
	struct kvp *pair;

	assert(key && value);

	// If the key already exists, overwrite its value, otherwise create a
	// new key/value pair.
	//
	pair = kvp_find(key);

	if (pair)
	{
		kvp_change_value(pair, value);
	}
	else
	{
		kvp_list = list_add(kvp_list, kvp_new(key, value));
	}
}


