/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <assert.h>
#include <string.h>

#include "display.h"
#include "list.h"

#include "defs.h"

#ifdef AUDIOBOX_ENABLE_LCDPROC
#  include "display_lcdproc.h"
#endif

#ifdef AUDIOBOX_ENABLE_CONSOLE
#  include "display_console.h"
#endif

// List of compiled in display drivers.
//
// NOTE: (void *) casts scare compiler warnings away. The compiler does not
// know that the objects referenced here are 'derived' from display_t and
// therefore gives a warning.
//
display_t *display_list[] = {
#ifdef AUDIOBOX_ENABLE_LCDPROC
	(void *) &lcd_display,
#endif
#ifdef AUDIOBOX_ENABLE_CONSOLE
	(void *) &console_display,
#endif
	NULL
};


static struct session_d
{
	screen_t *screen_stack[SCREEN_STACK_SIZE];
	int screen_stack_idx;
}
session[NUM_SESSION];

// Pointer to the current session structure
//
static struct session_d *sp = session;

/******************************************************************************/

void display_session_set(unsigned int idx)
{
	assert(idx < NUM_SESSION);

	sp = &session[idx];
}


/******************************************************************************/

unsigned int display_session_idx()
{
	return (unsigned int) (sp - session);
}


/******************************************************************************/

void display_push(screen_t * screen)
{
	// This is somewhat brutal, but well...
	//
	assert(sp->screen_stack_idx < SCREEN_STACK_SIZE);

	sp->screen_stack[sp->screen_stack_idx++] = screen;
}


/******************************************************************************/

void display_pop()
{
	// we never pop the first screen, because it is the main menu screen.
	//
	if (sp->screen_stack_idx > 1)
	{
		screen_t *current = sp->screen_stack[--sp->screen_stack_idx];

		if (current->delete)
		{
			(current->delete) (current);
		}
	}
}


/******************************************************************************/

screen_t *display_current_screen()
{
	return sp->screen_stack[sp->screen_stack_idx - 1];
}


/******************************************************************************/

void display_show()
{
	screen_t *screen = display_current_screen();
	int idx;
	display_t *display;

	assert(NULL != screen);

	// Before we can show the screens, we have to update the screen object,
	// otherwise the content will never change.
	//
	if (screen->update)
	{
		screen->update(screen);
	}


	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (display->show)
		{
			display->show(display, screen->ctx, screen->type);
		}
	}
}


/******************************************************************************/

void display_cmd(unsigned int cmd, void *arg)
{
	int idx;
	display_t *display;

	assert(NULL != display_current_screen());

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (display->cmd)
		{
			display->cmd(display, cmd, arg);
		}
	}
}


/******************************************************************************/

void display_open()
{
	int idx;
	display_t *display;

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (display->open)
		{
			display->open(display);
		}
	}
}


/******************************************************************************/

void display_close()
{
	int idx;
	display_t *display;

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (display->close)
		{
			display->close(display);
		}
	}
}


/******************************************************************************
 *
 * display_fds() collects all display fds and sets them in the variable fds.
 *
 * Returns the value of the highest fd collected.
 *
 ******************************************************************************/

int display_fds(fd_set * fds)
{
	int idx;
	int highest_fd = 0;
	display_t *display;

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		int fd;

		// skip displays that do not support fds
		//
		if (!display->fd)
			continue;

		fd = display->fd(display);
		FD_SET(fd, fds);

		if (fd > highest_fd)
			highest_fd = fd;
	}
	return highest_fd;
}


/******************************************************************************/

void display_handle_event(fd_set * fds)
{
	int idx;
	display_t *display;

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		// Skip displays that do not support fds
		//
		if (!display->fd)
			continue;


		if (FD_ISSET(display->fd(display), fds) && display->ev_handler)
		{
			display->ev_handler(display);
		}
	}
}


/******************************************************************************/

void display_handle_key(unsigned int key)
{
	screen_t *screen = display_current_screen();

	assert(NULL != screen);

	if (screen->handle_key)
	{
		screen->handle_key(screen, key);
	}
}


/******************************************************************************/

int display_init()
{
	int idx;
	int rc = 0;
	display_t *display;

	// Clear session structure
	//
	bzero(session, sizeof(session));

	// Open all displays
	//
	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		rc |= display->open(display);
	}
	return rc;
}
