/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include <assert.h>

#include "fileutil.h"
#include "player.h"

/******************************************************************************/

static struct player_registry_d
{
	int		file_type;
	player_if_t	*interface;
}
player_list[4];

static int last_entry_used = 0;

#define PLAYER_NUM_REGISTRY_ENTRIES	\
			(sizeof(player_list) / sizeof(struct player_registry_d))

/******************************************************************************/

int player_register(int file_type, player_if_t *interface)
{
	assert(NULL != interface);

	if (last_entry_used >= PLAYER_NUM_REGISTRY_ENTRIES)
	{
		return -1;
	}

	// We assume that players are nice and only register once.
	//
	player_list[last_entry_used].file_type	= file_type;
	player_list[last_entry_used].interface	= interface;

	last_entry_used++;
	return 0;
}


/******************************************************************************/

static player_if_t *player_get_if(int file_type)
{
	int i;

	for (i = 0; i < last_entry_used; ++i)
	{
		if (player_list[i].file_type == file_type)
		{
			return player_list[i].interface;
		}
	}

	// If we get here, we did not find the required file type player.
	//
	// However, we KNOW, that the player for type FILE_TYPE_UNKNOWN IS
	// registered. All it's functions return appropriate error codes. We
	// use its interface...
	//
	return player_get_if(FILE_TYPE_UNKNOWN);
}


/******************************************************************************/

static unsigned int current_type = FILE_TYPE_UNKNOWN;


/******************************************************************************/

int player_fd()
{
	return player_get_if(current_type)->fd();
}


/******************************************************************************/

int player_is_playing()
{
	return player_get_if(current_type)->is_playing();
}


/******************************************************************************/

int player_is_paused()
{
	return player_get_if(current_type)->is_paused();
}


/******************************************************************************/

int player_has_ended()
{
	return player_get_if(current_type)->has_ended();
}


/******************************************************************************/

const char *player_time()
{
	return player_get_if(current_type)->time();
}


/******************************************************************************/

const char *player_time_remaining()
{
	return player_get_if(current_type)->time_remaining();
}


/******************************************************************************/

int player_current_frame()
{
	return player_get_if(current_type)->current_frame();
}


/******************************************************************************/

int player_sample_rate()
{
	return player_get_if(current_type)->sample_rate();
}


/******************************************************************************/

int player_bit_rate()
{
	return player_get_if(current_type)->bit_rate();
}


/******************************************************************************/

void player_shutdown()
{
	int	i;

	current_type = FILE_TYPE_UNKNOWN;

	// We'll kill everything we have, just in case...
	//
	for (i = 0; i < last_entry_used; ++i)
	{
		player_list[i].interface->exit();
	}
}


/******************************************************************************/

void player_play(const char *path, unsigned int type)
{
	assert(NULL != path);

	// Launch the appropriate player for the song type (if not already
	// running).
	//
	if (current_type != type)
	{
		// Shut down current player.
		//
		player_get_if(current_type)->exit();

		// Initialize requested player.
		//
		player_get_if(type)->init();

		// Remember new player type.
		//
		current_type = type;
	}

	// Start playing the file.
	//
	player_get_if(current_type)->play(path);
}


/******************************************************************************/

int player_stop()
{
	return player_get_if(current_type)->stop();
}


/******************************************************************************/

int player_pause()
{
	return player_get_if(current_type)->pause();
}


/******************************************************************************/

void player_fwd()
{
	player_get_if(current_type)->fwd();
}


/******************************************************************************/

void player_rew()
{
	player_get_if(current_type)->rew();
}


/******************************************************************************/

void player_goto_frame(int frame)
{
	player_get_if(current_type)->goto_frame(frame);
}


/******************************************************************************/

int player_recv_message()
{
	return player_get_if(current_type)->recv_message();
}


/******************************************************************************/
