/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "keytab.h"
#include "info.h"
#include "menu.h"
#include "util.h"

#include "search.h"

/******************************************************************************/

static int search_obj_counter = 0;

/******************************************************************************/

typedef struct search_d
{
	// Screen base 'class' object
	//
	screen_t parent;

	// display context object
	//
	search_display_context_t ctx;

	list_t *search_result;	// current search result
	int search_mode_full;	// search mode flag


#define m_ctx                   ((search_t *) screen)->ctx
#define m_search_buf            ((search_t *) screen)->ctx.search_buf
#define m_idx                   ((search_t *) screen)->ctx.idx
#define m_search_mode           ((search_t *) screen)->ctx.search_mode
#define m_search_mode_full      ((search_t *) screen)->search_mode_full
#define m_search_result         ((search_t *) screen)->search_result
}
search_t;


/******************************************************************************/

static void search_set_search_result(screen_t * screen, list_t *list)
{
	if (m_search_result)
	{
		list_free(m_search_result);
	}
	m_search_result = list;
}


/******************************************************************************/

static list_t *search_do_search(screen_t * screen, const char *str)
{
	list_t *result;

	if (m_search_result && !m_search_mode_full)
	{
		result = file_entry_list_search(m_search_result, str);
	}
	else
	{
		result = file_entry_list_search_all(str);
	}
	return result;
}


/******************************************************************************/

static void search_update(screen_t * screen)
{
	assert((NULL != screen) && (TYPE_SEARCH == screen->type));


	// We clear the search_result if this is the first object.
	//
	if (1 == search_obj_counter)
	{
		search_set_search_result(screen, NULL);
	}

	// Update full/sub dearch flag
	//
	if (m_search_result && !m_search_mode_full)
	{
		m_search_mode = SEARCH_SUB;
	}
	else
	{
		m_search_mode = SEARCH_FULL;
	}
}


/******************************************************************************/

static int search_handle_key(screen_t * screen, int key)
{
	screen_t *info;
	list_t *result = NULL;

	assert((NULL != screen) && (TYPE_SEARCH == screen->type));

	switch (key)
	{
	case KEY_BACKSPACE:
		// break, if we are at position 0
		//
		if (0 == m_idx)
			break;

		m_search_buf[--m_idx] = '\0';
		break;

	case KEY_SELECT:
		// start search here...
		//
		result = search_do_search(screen, m_search_buf);

		// Break, if no result
		//
		if (!result)
		{
			// give user feedback
			//
			info = info_new("Nothing found for:", 2000);
			info_set_text(info, 1, m_search_buf);

			// Pop the search screen
			//
			display_pop();

			// And show the info
			//
			display_push(info);
			break;
		}

		search_set_search_result(screen, result);

		display_push(menu_new_from_filelist(list_copy(result)));
		break;

	case KEY_ESC:
		display_pop();
		break;

	case KEY_SEARCH:
		// Toggle search mode flag
		//
		m_search_mode_full = !m_search_mode_full;
		break;

	default:
		// break, if we are at the end of the buffer
		//
		if (MAX_NAME_LEN - 1 == m_idx)
			break;

		m_search_buf[m_idx++] = key;
		m_search_buf[m_idx] = '\0';
	}

	return KEY_HANDLED;
}


/******************************************************************************/

static void search_delete(screen_t * screen)
{
	assert((NULL != screen) && (TYPE_SEARCH == screen->type));

	// Free the search result list
	//
	search_set_search_result(screen, NULL);

	free(screen);

	search_obj_counter--;
}

/******************************************************************************/

screen_t *search_new(list_t *initial)
{
	screen_t *screen;

	// NOTE: We actually allocate a search_t object, but we handle it with
	// its parent 'class' (screen_t) pointer.
	//
	screen = (screen_t *) malloc(sizeof(search_t));
	assert(NULL != screen);

	bzero(screen, sizeof(search_t));
	screen->type		= TYPE_SEARCH;
	screen->update		= search_update;
	screen->delete		= search_delete;
	screen->handle_key 	= search_handle_key;
	screen->ctx		= &m_ctx;

	search_obj_counter++;
	search_set_search_result(screen, initial);

	return screen;
}
